from tests.test_resources import NemoResource
from mock import patch
from flask_nemo.plugins.default import Breadcrumb


class TestBreadcrumb(NemoResource):
    """ Test Breadcrumb default Plugins
    """

    def test_make_collection_breadcrumb_shorter(self):
        """ collection breadcrumb should include only collection not linked
        """
        with patch("requests.get", return_value=self.getCapabilities):
            make_breadcrumbs = Breadcrumb().render
            bc = make_breadcrumbs(
                lang="eng",
                collections={
                    'current': {
                        'model': 'http://w3id.org/dts-ontology/resource',
                        'label': 'Divān (English)',
                        'type': 'http://chs.harvard.edu/xmlns/cts/CTSCollection',
                        'id': 'urn:cts:farsiLit:hafez.divan.perseus-eng1'
                    },
                    'parents': [{
                        'model': 'http://w3id.org/dts-ontology/collection',
                        'size': 3,
                        'id': 'urn:cts:farsiLit:hafez.divan',
                        'type': 'http://chs.harvard.edu/xmlns/cts/CTSCollection',
                        'label': 'Divān'
                    }, {
                        'model': 'http://w3id.org/dts-ontology/collection',
                        'size': 1,
                        'id': 'urn:cts:farsiLit:hafez',
                        'type': 'http://chs.harvard.edu/xmlns/cts/CTSCollection',
                        'label': 'Hafez'
                    }]
                }
            )["breadcrumbs"]
            self.assertEqual(bc, [{
                'title': 'Text Collections',
                'args': {},
                'link': '.r_collections'
            }, {
                'title': 'Hafez',
                'args': {
                    'objectId': 'urn:cts:farsiLit:hafez',
                    'semantic': 'hafez'
                },
                'link': '.r_collection_semantic'
            }, {
                'title': 'Divān',
                'args': {
                    'objectId': 'urn:cts:farsiLit:hafez.divan',
                    'semantic': 'divan'
                },
                'link': '.r_collection_semantic'
            }, {
                'title': 'Divān (English)',
                'args': {},
                'link': None
            }])


    def test_make_collection_breadcrumb(self):
        """ collection breadcrumb should include only collection not linked
        """
        with patch("requests.get", return_value=self.getCapabilities):
            make_breadcrumbs = Breadcrumb().render
            bc = make_breadcrumbs(
                lang="eng",
                collections={
                    'current': {
                        'model': 'http://w3id.org/dts-ontology/resource',
                        'label': 'Divān (English)',
                        'type': 'http://chs.harvard.edu/xmlns/cts/CTSCollection',
                        'id': 'urn:cts:farsiLit:hafez.divan.perseus-eng1'
                    },
                    'parents': [{
                        'model': 'http://w3id.org/dts-ontology/collection',
                        'size': 3,
                        'id': 'urn:cts:farsiLit:hafez.divan',
                        'type': 'http://chs.harvard.edu/xmlns/cts/CTSCollection',
                        'label': 'Divān'
                    }, {
                        'model': 'http://w3id.org/dts-ontology/collection',
                        'size': 1,
                        'id': 'urn:cts:farsiLit:hafez',
                        'type': 'http://chs.harvard.edu/xmlns/cts/CTSCollection',
                        'label': 'Hafez'
                    }, {
                        'model': 'http://w3id.org/dts-ontology/collection',
                        'size': 1,
                        'id': 'urn:perseus:farsiLit',
                        'type': 'http://chs.harvard.edu/xmlns/cts/CTSCollection',
                        'label': 'Farsi'
                    }]
                }
            )["breadcrumbs"]
            self.assertEqual(bc, [{
                'title': 'Text Collections',
                'args': {},
                'link': '.r_collections'
            }, {
                'title': 'Farsi',
                'args': {
                    'objectId': 'urn:perseus:farsiLit',
                    'semantic': 'farsi'
                },
                'link': '.r_collection_semantic'
            }, {
                'title': 'Hafez',
                'args': {
                    'objectId': 'urn:cts:farsiLit:hafez',
                    'semantic': 'hafez'
                },
                'link': '.r_collection_semantic'
            }, {
                'title': 'Divān',
                'args': {
                    'objectId': 'urn:cts:farsiLit:hafez.divan',
                    'semantic': 'divan'
                },
                'link': '.r_collection_semantic'
            }, {
                'title': 'Divān (English)',
                'args': {},
                'link': None
            }])



from unittest import TestCase
from flask_nemo.plugin import PluginPrototype
from tests.test_plugin.test_resources import make_client


class PluginAssets(PluginPrototype):
    CSS = [
        "./tests/test_data/empty.css",
        "//foo.bar/test.css",
        "http://bar.foo/test.css",
        "https://super.secure/mypasswordin.css"
    ]
    JS = [
        "./tests/test_data/empty.js",
        "//foo.bar/test.js",
        "http://bar.foo/test.js",
        "https://super.secure/mypasswordin.js"
    ]
    STATICS = [
        "./tests/test_data/assets/fake.png"
    ]


class PluginClearAssets(PluginAssets):
    CLEAR_ASSETS = True
    STATIC_FOLDER = "tests/test_data/plugin_assets"


class TestPluginAssets(TestCase):
    """ Test plugin implementation of filters
    """

    def test_plugin_assets_in_templates(self):
        client = make_client(PluginAssets(name="One"))
        query_data = str(client.get("/").data)

        jss = [
            "//foo.bar/test.js",
            "http://bar.foo/test.js",
            "https://super.secure/mypasswordin.js"
        ]
        assert_length_js = len(jss)
        for js in jss:
            self.assertIn(
                '<script src="{0}"></script>'.format(js), query_data,
                "Templates should correctly link external js"
            )
            assert_length_js -= 1
        self.assertEqual(assert_length_js, 0, "All js file should have been checked")

        csss = [
            "//foo.bar/test.css",
            "http://bar.foo/test.css",
            "https://super.secure/mypasswordin.css"
        ]
        assert_length_js = len(csss)
        for css in csss:
            self.assertIn(
                '<link rel="stylesheet" href="{0}">'.format(css), query_data,
                "Templates should correctly link external css"
            )
            assert_length_js -= 1
        self.assertEqual(assert_length_js, 0, "All css files should have been checked")

        self.assertIn(
            '<link rel="stylesheet" href="/assets/nemo.secondary/css/empty.css">', query_data,
            "Secondary Internal CSS should be linked"
        )
        self.assertIn(
            '<script src="/assets/nemo.secondary/js/empty.js"></script>', query_data,
            "Secondary Internal JS should be retrieved"
        )

    def test_check_original_status(self):
        """ This test just check that by default, the value we use are not in the directory
        """
        client = make_client()
        query_data = str(client.get("/").data)
        self.assertNotIn(
            '<script src="//foo.bar/test.js"></script>', query_data,
            "Templates should not have it by default"
        )

    def test_serving_assets(self):
        """ Ensure plugins assets are served
        """
        client = make_client(PluginAssets())
        query_data = str(client.get("/assets/nemo.secondary/css/empty.css").data)
        self.assertIn(".empty {", query_data, "Local CSS File should be read")

        query_data = str(client.get("/assets/nemo.secondary/js/empty.js").data)
        self.assertIn("var empty = True;", query_data, "Local JS File should be read")

        query_data = str(client.get("/assets/nemo.secondary/static/fake.png").data)
        self.assertIn("fake", query_data, "Local Static should be read")

    def test_clearing_assets(self):
        client = make_client(PluginClearAssets())
        query_data = str(client.get("/assets/nemo.secondary/css/empty.css").data)
        self.assertIn(".empty {", query_data, "Local Secondary CSS File should be read")

        query_data = str(client.get("/assets/nemo.secondary/js/empty.js").data)
        self.assertIn("var empty = True;", query_data, "Local Secondary JS File should be read")

        query_data = str(client.get("/assets/nemo/plugin.css").data)
        self.assertIn(".pluginstuff", query_data, "Primary Assets should be served when overwritting")

        self.assertEqual(
            client.get("/assets/nemo/css/theme.min.css").status_code, 404,
            "Old Primary Assets should not be served when overwritting"
        )

    def test_prevent_clearing_assets(self):
        client = make_client(PluginClearAssets(), prevent_plugin_clearing_assets=True)
        query_data = str(client.get("/assets/nemo/css/theme.min.css").data)
        self.assertIn("body, html", query_data, "Nemo Primary Assets should be not served when prevent is on")

        self.assertEqual(
            client.get("/assets/nemo/plugin.css").status_code, 404,
            "Plugin Primary Assets should not be served when prevent is on"
        )



from flask import Flask
from flask_nemo import Nemo
from tests.test_resources import NautilusDummy
from flask_nemo.chunker import level_grouper


def make_client(*args, **kwargs):
    app = Flask("Nemo")
    app.debug = True
    if len(args):
        nemo = Nemo(
            app=app,
            base_url="",
            resolver=NautilusDummy,
            chunker={"default": lambda x, y: level_grouper(x, y, groupby=30)},
            plugins=list(args),
            **kwargs
        )
    else:
        nemo = Nemo(
            app=app,
            base_url="",
            resolver=NautilusDummy,
            chunker={"default": lambda x, y: level_grouper(x, y, groupby=30)},
            plugins=None,
            **kwargs
        )
    return app.test_client()


# -*- coding: utf-8 -*-


class QueryPrototype(object):
    """ Prototype for Nemo Query API Implementations

    :param name: The Name of this Query API
    :type name: str
    :param getreffs: callback function to retrieve a list of references given URN
    :type getreffs: function

    """

    MATCH_EXACT = "."
    MATCH_LOWER = ".%"
    MATCH_HIGHER = "%."
    MATCH_RANGE = "-"
    MATCH_ALL = "%.%"

    def __init__(self, getreffs, **kwargs):
        self.__getreffs__ = getreffs

    def getAnnotations(self, targets, wildcard=".", include=None, exclude=None, limit=None, start=1, expand=False,
                       **kwargs):
        """ Retrieve annotations from the query provider

        :param targets: The CTS URN(s) to query as the target of annotations
        :type targets: [MyCapytain.common.reference.URN], URN or None
        :param wildcard: Wildcard specifier for how to match the URN
        :type wildcard: str
        :param include: URI(s) of Annotation types to include in the results
        :type include: list(str)
        :param exclude: URI(s) of Annotation types to include in the results
        :type exclude: list(str)
        :param limit: The max number of results to return (Default is None for no limit)
        :type limit: int
        :param start: the starting record to return (Default is 1)
        :type start: int 
        :param expand: Flag to state whether Annotations are expanded (Default is False)
        :type expand: bool
    
        :return: Tuple representing the query results. The first element
                 The first element is the number of total Annotations found
                 The second element is the list of Annotations
        :rtype: (int, list(Annotation)

        .. note::

            Wildcard should be one of the following value

            - '.' to match exact,
            - '.%' to match exact plus lower in the hierarchy
            - '%.' to match exact + higher in the hierarchy
            - '-' to match in the range
            - '%.%' to match all

        """
        return 0, []

    def getResource(self, sha):
        """ Retrieve a single annotation resource by sha

        :param sha: The sha of the resource
        :type sha: str
        :return: the requested annotation resource
        :rtype: AnnotationResource
        """
        return None



"""

    Test implementation of customization through init for example

"""

from tests.test_resources import NemoResource
from flask_nemo import Nemo
from MyCapytain.resources.collections.cts import Text
from lxml import etree
from tests.test_resources import NautilusDummy


class TestCustomizer(NemoResource):
    """ Test customization appliers
    """
    def test_chunker_default(self):
        """ Test that the chunker default is called and applied
        """
        def default(text, reffs):
            self.assertEqual(str(text.urn), "urn:cts:phi1294.phi002.perseus-lat2")
            self.assertEqual(reffs, ["1.pr"])
            return [("1.pr", "I PR")]

        nemo = Nemo(chunker={
            "default": default
        })
        chunked = nemo.chunk(
            Text(
                urn="urn:cts:phi1294.phi002.perseus-lat2"
            ),
            ["1.pr"]
        )
        self.assertEqual(chunked, [("1.pr", "I PR")])

    def test_chunker_urn(self):
        """ Test that the chunker by urn is called and applied
        """
        def urn(text, reffs):
            self.assertEqual(str(text.urn), "urn:cts:phi1294.phi002.perseus-lat2")
            self.assertEqual(reffs, ["1.pr"])
            return [("1.pr", "I PR")]

        nemo = Nemo(chunker={
            "default": lambda x, y: y,
            "urn:cts:phi1294.phi002.perseus-lat2": urn
        })
        chunked = nemo.chunk(
            Text(
                urn="urn:cts:phi1294.phi002.perseus-lat2"
            ),
            ["1.pr"]
        )
        self.assertEqual(chunked, [("1.pr", "I PR")])

    def test_transform_default_function(self):
        """ Test that the transform default is called and applied when it's a function
        """
        urn_given = "urn:cts:latinLit:phi1294.phi002.perseus-lat2"
        ref_given = "1.pr.1"

        def default(work, xml, objectId, subreference=None):
            self.assertEqual(str(work.urn), "urn:cts:latinLit:phi1294.phi002.perseus-lat2")
            self.assertEqual(objectId, urn_given, "Passage URN should be passed to transform")
            self.assertEqual(subreference, ref_given, "Passage URN should be passed to transform")
            self.assertEqual(xml, "<a></a>")
            return "<b></b>"

        nemo = Nemo(transform={
            "default": default
        })
        transformed = nemo.transform(
            Text(
                urn="urn:cts:latinLit:phi1294.phi002.perseus-lat2"
            ),
            "<a></a>",
            urn_given,
            ref_given
        )
        self.assertEqual(transformed, "<b></b>")

    def test_transform_default_none(self):
        """ Test that the transform default is called and applied
        """
        nemo = Nemo()
        transformed = nemo.transform(
            Text(
                urn="urn:cts:phi1294.phi002.perseus-lat2"
            ),
            etree.fromstring("<a/>"),
            "urn:cts:latinLit:phi1294.phi002.perseus-lat2:1.pr.1"
        )
        self.assertEqual(transformed, "<a/>")

    def test_transform_urn_xslt(self):
        """ Test that the transform default is called and applied
        """

        nemo = Nemo(transform={
            "default": "tests/test_data/xsl_test.xml"
        })
        transformed = nemo.transform(
            Text(
                urn="urn:cts:latinLit:phi1294.phi002.perseus-lat2"
            ),
            etree.fromstring('<tei:body xmlns:tei="http://www.tei-c.org/ns/1.0" />'),
            objectId="urn:cts:latinLit:phi1294.phi002.perseus-lat2",
            subreference="1.pr.1"
        )
        self.assertEqual(transformed, '<tei:notbody xmlns:tei="http://www.tei-c.org/ns/1.0"></tei:notbody>',
            "It should autoclose the tag"
        )

    def test_transform_match(self):
        """ Test that the transform default is called and applied
        """

        nemo = Nemo(transform={
            "default": lambda x: self.assertEqual(False, True, "This should not be run"),
            "urn:cts:latinLit:phi1294.phi002.perseus-lat2": "tests/test_data/xsl_test.xml"
        })
        transformed = nemo.transform(
            Text(
                urn="urn:cts:latinLit:phi1294.phi002.perseus-lat2"
            ),
            etree.fromstring('<tei:body xmlns:tei="http://www.tei-c.org/ns/1.0" />'),
            objectId="urn:cts:latinLit:phi1294.phi002.perseus-lat2",
            subreference="1.pr.1"
        )
        self.assertEqual(transformed, '<tei:notbody xmlns:tei="http://www.tei-c.org/ns/1.0"></tei:notbody>',
            "It should autoclose the tag"
        )



import flask_nemo._data
from flask_nemo.common import getFromDict
from collections import OrderedDict
from slugify import slugify
from operator import itemgetter


def f_slugify(string):
    """ Slugify a string

    :param string: String to slugify
    :return: Slugified string
    """
    return slugify(string)


def f_formatting_passage_reference(string):
    """ Get the first part only of a two parts reference

    :param string: A urn reference part
    :type string: str
    :return: First part only of the two parts reference
    :rtype: str
    """
    return string.split("-")[0]


def f_i18n_iso(isocode, lang="eng"):
    """ Replace isocode by its language equivalent

    :param isocode: Three character long language code
    :param lang: Lang in which to return the language name
    :return: Full Text Language Name
    """
    if lang not in flask_nemo._data.AVAILABLE_TRANSLATIONS:
        lang = "eng"

    try:
        return flask_nemo._data.ISOCODES[isocode][lang]
    except KeyError:
        return "Unknown"


def f_order_resource_by_lang(versions_list):
    """ Takes a list of versions and put translations after editions

    :param versions_list: List of text versions
    :type versions_list: [Text]
    :return: List where first members will be editions
    :rtype: [Text]
    """
    return sorted(versions_list, key=itemgetter("lang"))


def f_hierarchical_passages(reffs, citation):
    """ A function to construct a hierarchical dictionary representing the different citation layers of a text

    :param reffs: passage references with human-readable equivalent
    :type reffs: [(str, str)]
    :param citation: Main Citation
    :type citation: Citation
    :return: nested dictionary representing where keys represent the names of the levels and the final values represent the passage reference
    :rtype: OrderedDict
    """
    d = OrderedDict()
    levels = [x for x in citation]
    for cit, name in reffs:
        ref = cit.split('-')[0]
        levs = ['%{}|{}%'.format(levels[i].name, v) for i, v in enumerate(ref.split('.'))]
        getFromDict(d, levs[:-1])[name] = cit
    return d


def f_is_str(value):
    """ Check if object is a string

    :param value: object to check against
    :return: Return if value is a string
    """
    return isinstance(value, str)


def f_i18n_citation_type(string, lang="eng"):
    """ Take a string of form %citation_type|passage% and format it for human

    :param string: String of formation %citation_type|passage%
    :param lang: Language to translate to
    :return: Human Readable string

    .. note :: To Do : Use i18n tools and provide real i18n
    """
    s = " ".join(string.strip("%").split("|"))
    return s.capitalize()


def f_annotation_filter(annotations, type_uri, number):
    """ Annotation filtering filter

    :param annotations: List of annotations
    :type annotations: [AnnotationResource]
    :param type_uri: URI Type on which to filter
    :type type_uri: str
    :param number: Number of the annotation to return
    :type number: int
    :return: Annotation(s) matching the request
    :rtype: [AnnotationResource] or AnnotationResource
    """
    filtered = [
        annotation
        for annotation in annotations
        if annotation.type_uri == type_uri
    ]
    number = min([len(filtered), number])
    if number == 0:
        return None
    else:
        return filtered[number-1]



from unittest import TestCase
from flask_nemo.query.annotation import AnnotationResource, Target
from MyCapytain.common.reference import URN
from flask_nemo.query.resolve import Resolver
import json


class RetrieverMock(object):
    @staticmethod
    def match(uri):
        return True

    @staticmethod
    def read(uri):
        return "{} has been read".format(uri), "mimetype"


class WTarget(Target):
    def __init__(self, param_dict):
        super(WTarget, self).__init__(objectId=param_dict["urn"])


class TestTarget(TestCase):
    """ Test method / values specific to the basic Target implementation
    """

    def setUp(self):
        self.alias = Target
        self.target_1 = "urn:cts:latinLit:phi1294.phi002.perseus-lat2"
        self.target_1_export = {"source":"urn:cts:latinLit:phi1294.phi002.perseus-lat2"}
        self.target_2 = URN("urn:cts:latinLit:phi1294.phi002.perseus-lat2:1.1.1")
        self.target_2_export = {
            "source": "urn:cts:latinLit:phi1294.phi002.perseus-lat2",
            "selector": {
                "type": "FragmentSelector",
                "conformsTo": "http://ontology-dts.org/terms/subreference",
                "value": "1.1.1"
            }
        }
        self.target_3 = "urn:cts:latinLit:phi1294.phi002.perseus-lat2:1.1.1"
        self.target_3_export = {
            "source": "urn:cts:latinLit:phi1294.phi002.perseus-lat2:1.1.1"
        }

    def test_init(self):
        """ Assert initation takes into account params
        """
        self.assertEqual(
            self.alias("urn:cts:latinLit:phi1294.phi002.perseus-lat2").objectId,
            "urn:cts:latinLit:phi1294.phi002.perseus-lat2",
            "String are taken as parameter"
        )
        self.assertEqual(
            self.alias(URN("urn:cts:latinLit:phi1294.phi002.perseus-lat2")).objectId,
            "urn:cts:latinLit:phi1294.phi002.perseus-lat2",
            "URN are taken as parameter"
        )

    def test_to_json(self):
        """ Assert json-serializable is produced by the given resource is produced
        """
        self.assertEqual(
            json.loads(json.dumps(self.alias(self.target_1).to_json())),
            self.target_1_export,
            "JSON of basic target is a plain string"
        )
        self.assertEqual(
            json.loads(json.dumps(self.alias(self.target_2).to_json())),
            self.target_2_export,
            "JSON of basic target is a plain string"
        )


class TestAnnotationResource(TestCase):
    def setUp(self):
        self.resolver = Resolver(
            RetrieverMock
        )

        self.params_1 = [
            "http://localhost",
            URN("urn:cts:latinLit:phi1294.phi002.perseus-lat2"),
            "http://foo.bar/treebank",
            self.resolver,
            Target,
            "application/xml",
            "treebank"
        ]
        self.params_2 = [
            "http://localhost1",
            {"urn": URN("urn:cts:latinLit:phi1294.phi002.perseus-lat2")},
            "http://foo.bar/treebank1",
            self.resolver,
            WTarget,
            "application/xml1",
            "treebank1"
        ]

    def test_init_and_properties(self):
        """ Ensure init values are taken into account
        """
        anno = AnnotationResource(
            *self.params_1
        )
        self.assertEqual(
            anno.mimetype, "application/xml",
            "Mimetype should be set with init params"
        )
        self.assertEqual(
            anno.slug, "treebank",
            "Slug should be set with init params"
        )
        self.assertEqual(
            anno.sha, "a076083ce9233ea6bb5263109d05d0780261c992c83c0a5787d79d9f62c71266",
            "Sha should be set with init params"
        )
        self.assertEqual(
            anno.type_uri, "http://foo.bar/treebank",
            "Type URI should be set with init params"
        )
        self.assertEqual(
            anno.uri, "http://localhost",
            "URI should be set with init params"
        )
        self.assertEqual(
            str(anno.target.objectId), "urn:cts:latinLit:phi1294.phi002.perseus-lat2",
            "Target should be set with init params"
        )
        self.assertIsInstance(
            anno.target, Target,
            "Target alias should be used"
        )
        self.assertEqual(
            anno.expandable, False,
            "Default annotation resource are not expandable"
        )

    def test_target_alias(self):
        """ Ensure alias class can be overwritten
        """
        anno = AnnotationResource(
            *self.params_2
        )
        self.assertIsInstance(
            anno.target, WTarget,
            "Target should be built through annotation alias class target_class param"
        )

    def test_target_expand(self):
        """ Ensure existing method and default behaviour
        """
        anno = AnnotationResource(
            *self.params_2
        )
        self.assertEqual(
            anno.expand(), [],
            "Default annotation should expand to an eempty list"
        )

    def test_read(self):
        """ Ensure existing method and default behaviour

        See the mocked resolver at the beginning of the file
        """
        anno = AnnotationResource(
            *self.params_2
        )
        self.assertEqual(
            anno.read(), anno.uri+" has been read",
            "Default annotation should expand to an empty list"
        )
        self.assertEqual(
            anno.mimetype, "mimetype",
            ".read() should update mimetype"
        )



from pkg_resources import resource_filename
import json

with open(resource_filename("flask_nemo", "data/i18n/languages.json")) as f:
    ISOCODES = json.load(f)
    AVAILABLE_TRANSLATIONS = set([key for value in ISOCODES.values() for key in value.keys()])



from flask_nemo.query.resolve import Resolver, CTSRetriever, HTTPRetriever, LocalRetriever, UnresolvableURIError
from tests.test_resources import NautilusDummy
from unittest import TestCase
from mock import patch


def mocked_response(content, mime):
    class MockResponse(object):
        def __init__(self, data, mime="text"):
            self.data = data
            self.mime = mime

        @property
        def text(self):
            return self.data

        @property
        def content(self):
            return self.data

        @property
        def headers(self):
            return {
                "Content-Type": self.mime
            }

    return MockResponse(content, mime)


class TestCTSRetrievers(TestCase):
    def test_matching(self):
        """ Test the CTS retriever matching
        """
        tests = [
            ("urn:cts:latinLit:phi1294:phi002.perseus-lat2", True),
            ("https://foo.com/bar", False)
        ]
        i = len(tests)
        for uri, matching in tests:
            i -= 1
            if matching:
                msg = "CTS URNs should match"
            else:
                msg = "URI which are not cts urn should not match"

            self.assertEqual(
                CTSRetriever(NautilusDummy).match(uri), matching,
                msg
            )

        self.assertEqual(
            i, 0,
            "All tests should have been run"
        )

    def test_retrieve_resource(self):
        """ Ensure CTSRetriever actually get resource
        """
        ret = CTSRetriever(NautilusDummy)
        passage, mime = ret.read("urn:cts:latinLit:phi1294.phi002:1.pr.1")
        self.assertEqual(mime, "text/xml", "Mime type of CTS should be text/xml")
        self.assertIn(
            '<TEI xmlns="http://www.tei-c.org/ns/1.0">', passage,
            "TEI is visible as a tag"
        )
        self.assertIn(
            "Spero me secutum in libellis meis tale temperamen", passage,
            "Text of the passage is retrieved"
        )


class TestHTTPRetrievers(TestCase):
    """ Tests for the HTTP retriever
    """

    def test_matching(self):
        """ Test the HTTP retriever matching
        """
        tests = [
            ("urn:cts:latinLit:phi1294:phi002.perseus-lat2", False),
            ("https://foo.com/bar", True)
        ]
        i = len(tests)
        for uri, matching in tests:
            i -= 1
            if matching:
                msg = "HTTP should match"
            else:
                msg = "URI which are not HTTP should not match"

            self.assertEqual(
                HTTPRetriever().match(uri), matching,
                msg
            )

        self.assertEqual(
            i, 0,
            "All tests should have been run"
        )

    def test_retrieve_resource(self):
        """ Ensure HTTPRetriever actually get resource
        """
        ret = HTTPRetriever()
        uris = [
            ("http://foo.bar/com", "I am some content", "application/html")
        ]
        i = len(uris)
        for uri, content, mime in uris:
            i -= 1
            with patch("flask_nemo.query.resolve.request", return_value=mocked_response(content, mime)) as request:
                data, mimetype = ret.read(uri)
                request.assert_called_with("GET", uri)
                self.assertEqual(
                    data, content,
                    "Content should be read correctly"
                )
                self.assertEqual(mimetype, "application/html", "Mime is correctly read from headers")

        self.assertEqual(
            i, 0,
            "All tests should have been run"
        )


class TestLocalRetrievers(TestCase):
    """ Tests for the Local retriever
    """

    def test_matching(self):
        """ Test the Local retriever matching
        """
        localretriever = LocalRetriever(path="./tests/test_data")
        tests = [
            ("urn:cts:latinLit:phi1294:phi002.perseus-lat2", False, "URN should not match"),
            ("http://foo.com/bar", False, "URL should not match"),
            ("../test_data/empty.js", True, "File match and exists"),
            ("assets/fake.png", True, "File match and exists"),
            ("./something.html", False, "File is not in path"),
            ("../setup.py", False, "File out of path should not match"),
            ("something.html", False, "File does not exist")
        ]
        i = len(tests)
        for uri, matching, msg in tests:
            i -= 1

            self.assertEqual(
                localretriever.match(uri), matching,
                msg
            )

        self.assertEqual(
            i, 0,
            "All tests should have been run"
        )

    def test_retrieve_resource(self):
        """ Ensure LocalRetriever actually get resource
        """
        ret = LocalRetriever(path="./tests/test_data")
        uris = [
            ("../test_data/empty.js", "var empty = True;", "application/javascript", False),
            ("assets/fake.png", "fake", "image/png", True),
        ]
        i = len(uris)
        for uri, content, mime, sendfile in uris:
            i -= 1
            if sendfile:
                with patch("flask_nemo.query.resolve.send_file", return_value=content) as patched:
                    data, mimetype = ret.read(uri)
                    patched.assert_called_with(ret.__absolute__(uri))
            else:
                data, mimetype = ret.read(uri)

            self.assertIn(
                data, content,
                "Content of the file should be read"
            )

            self.assertEqual(
                mimetype, mime,
                "Mime should be correctly guessed"
            )

        self.assertEqual(
            i, 0,
            "All tests should have been run"
        )


class TestResolverWithRetriever(TestCase):
    """ Ensure retrievers stacks well
    """
    def setUp(self):
        self.resolver = Resolver(
            CTSRetriever(resolver=NautilusDummy),
            HTTPRetriever(),
            LocalRetriever(path="./tests/test_data")
        )

    def test_stack(self):
        uris = [
            ("urn:cts:latinLit:phi1294.phi002.perseus-lat2", "CTS Retriever should be resolved", CTSRetriever),
            ("http://foo.com/bar", "HTTP retriever should match the URI", HTTPRetriever),
            ("../test_data/empty.js", "Local Retriever should match complex URIs", LocalRetriever),
            ("assets/fake.png", "Local Retriever should match simple URIs", LocalRetriever)
        ]
        i = len(uris)
        for uri, msg, type_retriever in uris:
            self.assertEqual(
                type(self.resolver.resolve(uri)), type_retriever,
                msg
            )
            i -= 1
        self.assertEqual(i, 0, "All tests have been run")

    def test_stack_fails(self):
        """ Ensure that it still raises for unaccepted
        """
        with self.assertRaises(UnresolvableURIError, msg="Not safe file should fail"):
            self.resolver.resolve("../setup.py")



import os.path as op
from collections import OrderedDict
import re
from functools import reduce


""" Regular expression to match common literature namespace
"""
regMatch = re.compile("^[a-z]{3}Lit$")


def resource_qualifier(resource):
    """ Split a resource in (filename, directory) tuple with taking care of external resources

    :param resource: A file path or a URI
    :return: (Filename, Directory) for files, (URI, None) for URI
    """
    if resource.startswith("//") or resource.startswith("http"):
        return resource, None
    else:
        return reversed(op.split(resource))

ASSETS_STRUCTURE = {
    "js": OrderedDict(),
    "css": OrderedDict(),
    "static": OrderedDict()
}


def join_or_single(start, end):
    """ Join passages range. If they are the same, return a single part of the range

    :param start: Start of the passage range
    :param end: End of the passage range
    :return: Finale Passage Chunk Identifier
    """
    if start == end:
        return start
    else:
        return "{}-{}".format(
            start,
            end
        )


def getFromDict(dataDict, keyList):
    """Retrieves and creates when necessary a dictionary in nested dictionaries

    :param dataDict: a dictionary
    :param keyList: list of keys
    :return: target dictionary
    """
    return reduce(create_hierarchy, keyList, dataDict)


def create_hierarchy(hierarchy, level):
    """Create an OrderedDict

    :param hierarchy: a dictionary
    :param level: single key
    :return: deeper dictionary
    """
    if level not in hierarchy:
        hierarchy[level] = OrderedDict()
    return hierarchy[level]



from unittest import TestCase
from flask_nemo.plugin import PluginPrototype
from tests.test_plugin.test_resources import make_client
import werkzeug.routing
from copy import deepcopy as copy


class PluginRoute(PluginPrototype):
    ROUTES = [
        ("/doubletext/<object1>/<object2>/<subreference>", "r_double", ["GET"])
    ]

    TEMPLATES = {
        "plugin_route": "tests/test_data/plugin_templates_main/plugin"
    }
    ROUTE_TEMPLATES = {
        "r_double": "plugin_route::r_double.html"
    }
    HAS_AUGMENT_RENDER = True

    def r_double(self, object1, object2, subreference):
        args = self.nemo.r_passage(objectId=object1, subreference=subreference)
        # Call with other identifiers and add "visavis_" front of the argument
        args.update({
            "visavis_{0}".format(key): value for key, value in self.nemo.r_passage(
                object2, subreference
            ).items()
        })
        args["template"] = type(self).ROUTE_TEMPLATES["r_double"]
        return args

    def render(self, **kwargs):
        kwargs["lang"] = "fre"
        if "collections" in kwargs:
            kwargs["collections"]["current"]["label"] = "Other label"
        return kwargs


class PluginClearRoute(PluginRoute):
    CLEAR_ROUTES = True

    ROUTE_TEMPLATES = {
        "r_double": "plugin_cleared_route::r_double.html"
    }
    TEMPLATES = {
        "plugin_cleared_route": "tests/test_data/plugin_templates_main/plugin",
        "main": "tests/test_data/plugin_templates_main/main"
    }


class TestPluginRoutes(TestCase):
    """ Test plugin implementation of filters
    """

    def setUp(self):
        self.plugin_normal = PluginRoute(name="normal")
        self.plugin_namespacing = PluginRoute(name="test", namespacing=True)
        self.plugin_autonamespacing = PluginRoute(namespacing=True)

    def test_kwargs(self):
        self.assertEqual(self.plugin_normal.render(**dict(test=1, two=2)), {'lang': 'fre', "test": 1, "two": 2})

    def test_kwargs_proto(self):
        self.assertEqual(PluginPrototype().render(**dict(test=1, two=2)), {"test": 1, "two": 2})

    def test_page_works(self):
        """ Test that passage page contains what is relevant : text and next passages"""
        query_data = make_client(self.plugin_normal)\
            .get("/doubletext/urn:cts:farsiLit:hafez.divan.perseus-eng1/urn:cts:farsiLit:hafez.divan.perseus-ger1/1.1")\
            .data.decode()
        self.assertIn(
            'Ho ! Saki, pass around and offer the bowl (of love for God)', query_data,
            "English text should be displayed"
        )
        self.assertIn(
            'Finstere Schatten der Nacht!', query_data,
            "German text should be displayed"
        )

    def test_plugin_change_render(self):
        """ Check that the augmenting render function works and is correctly plugged
        """
        query_data = make_client(self.plugin_normal)\
            .get("/text/urn:cts:farsiLit:hafez.divan.perseus-eng1/passage/1.1")\
            .data.decode()
        self.assertIn(
            'Other label', query_data,
            "French Translation should be displayed"
        )

    def test_plugin_clear(self):
        """ Test that passage page contains what is relevant : text and next passages """
        plugin = PluginClearRoute(name="normal")
        client = make_client(plugin)

        req = client.get("/text/urn:cts:farsiLit:hafez.divan.perseus-eng1/passage/1.1")
        self.assertEqual(
            404, req.status_code,
            "Original routes should not exist anymore"
        )
        query_data = make_client(self.plugin_normal)\
            .get("/doubletext/urn:cts:farsiLit:hafez.divan.perseus-eng1/urn:cts:farsiLit:hafez.divan.perseus-ger1/1.1")\
            .data.decode()
        self.assertIn(
            'Ho ! Saki, pass around and offer the bowl (of love for God)', query_data,
            "English text should be displayed"
        )
        self.assertIn(
            'Finstere Schatten der Nacht!', query_data,
            "German text should be displayed"
        )

    def test_plugin_clear_with_error(self):
        """
            When clearing routes, we need to ensure Flask fails to run
            if there is still some other routes call
        """
        class TempPlugin(PluginClearRoute):
            TEMPLATES = copy(PluginRoute.TEMPLATES)
            ROUTE_TEMPLATES = copy(PluginRoute.ROUTE_TEMPLATES)

        plugin = TempPlugin(name="normal")
        client = make_client(plugin)

        with self.assertRaises(
                werkzeug.routing.BuildError,
                msg="Call to other routes in templates should fail to build"
        ):
            client.get("/doubletext/urn:cts:farsiLit:hafez.divan.perseus-eng1/urn:cts:farsiLit:hafez.divan.perseus-ger1/1.1")

    def test_template_overwrite(self):
        """ Checks that overwriting original templates works
        """
        class PluginTemplate(PluginPrototype):
            TEMPLATES = {
                "main": "tests/test_data/plugin_templates_main/main"
            }
        client = make_client(PluginTemplate())
        query_data = client.get("/text/urn:cts:farsiLit:hafez.divan.perseus-eng1/passage/1.1").data.decode()
        self.assertIn(
            'I am A CONTAINER !', query_data,
            "Container should have been overwritten"
        )
        query_data = client.get("/collections/urn:cts:farsiLit:hafez").data.decode()
        self.assertIn(
            'I am A CONTAINER !', query_data,
            "Container should have been overwritten on a second page as well"
        )



from flask import Flask
from flask_nemo import Nemo


def make_client(*args, **kwargs):
    app = Flask("Nemo")
    app.debug = True
    if len(args):
        nemo = Nemo(
            app=app,
            base_url="",
            plugins=list(args),
            **kwargs
        )
    else:
        nemo = Nemo(
            app=app,
            base_url="",
            plugins=None,
            **kwargs
        )
    return app.test_client()



from requests import request
import re
from os import path as op
from mimetypes import guess_type
from flask import send_file
from MyCapytain.common.constants import Mimetypes


class UnresolvableURIError(Exception):
    """ Error to be run when a URI is not resolvable
    """


class Resolver(object):

    """ Prototype for a Resolver
    :param retriever: Retriever(s) to use to resolve resources passed to this resolver
    :type retriever: Retriever instances
    """

    def __init__(self, *retrievers, **kwargs):
        self.__retrievers__ = retrievers

    def resolve(self, uri):
        """ Resolve a Resource identified by URI
        :param uri: The URI of the resource to be resolved
        :type uri: str
        :return: the contents of the resource as a string
        :rtype: str
        """
        for r in self.__retrievers__:
            if r.match(uri):
                return r
        raise UnresolvableURIError()


class RetrieverPrototype(object):

    """ Prototype for a Retriever
    """

    def match(self, uri):
        """ Check to see if this URI is retrievable by this Retriever implementation
        :param uri: the URI of the resource to be retrieved
        :type uri: str
        :return: True if it can be, False if not
        :rtype: bool
        """
        # prototype implementation can't retrieve anything!
        return False

    def read(self, uri):
        """ Retrieve the contents of the resource
        :param uri: the URI of the resource to be retrieved
        :type uri: str
        :return: the contents of the resource and it's mime type in a tuple
        :rtype: str, str
        """
        return None, "text/xml"


class HTTPRetriever(RetrieverPrototype):
    """ Http retriever retrieves resources being remotely hosted in CTS
    """
    __reg_exp__ = re.compile("^(https?:)?\/\/")

    def match(self, uri):
        """ Check to see if this URI is retrievable by this Retriever implementation

        :param uri: the URI of the resource to be retrieved
        :type uri: str
        :return: True if it can be, False if not
        :rtype: bool
        """
        # prototype implementation can't retrieve anything!
        return HTTPRetriever.__reg_exp__.match(uri) is not None

    def read(self, uri):
        """ Retrieve the contents of the resource

        :param uri: the URI of the resource to be retrieved
        :type uri: str
        :return: the contents of the resource
        :rtype: str
        """
        req = request("GET", uri)
        return req.content, req.headers['Content-Type']


class LocalRetriever(RetrieverPrototype):
    """ Http retriever retrieves resources being remotely hosted in CTS

    .. note:: Local Retriever needs to be instantiated

    :cvar FORCE_MATCH: Force the local retriever to read a resource even if it does not match with the regular expression
    :type FORCE_MATCH: bool
    """

    def __init__(self, path="./"):
        self.__path__ = op.abspath(path)

    def __absolute__(self, uri):
        """ Get the absolute uri for a file

        :param uri: URI of the resource to be retrieved
        :return: Absolute Path
        """
        return op.abspath(op.join(self.__path__, uri))

    def match(self, uri):
        """ Check to see if this URI is retrievable by this Retriever implementation

        :param uri: the URI of the resource to be retrieved
        :type uri: str
        :return: True if it can be, False if not
        :rtype: bool
        """
        absolute_uri = self.__absolute__(uri)

        return absolute_uri.startswith(self.__path__) and op.exists(absolute_uri)

    def read(self, uri):
        """ Retrieve the contents of the resource

        :param uri: the URI of the resource to be retrieved
        :type uri: str
        :return: the contents of the resource
        :rtype: str
        """
        uri = self.__absolute__(uri)
        mime, _ = guess_type(uri)
        if "image" in mime:
            return send_file(uri), mime
        else:
            with open(uri, "r") as f:
                file = f.read()
        return file, mime


class CTSRetriever(RetrieverPrototype):
    """ CTS retriever retrieves resources being remotely hosted in CTS

    .. note:: Local Retriever needs to be instantiated

    :param resolver: CTS5 Resolver
    :type resolver: MyCapytain.resolver.cts.*
    """
    __reg_exp__ = re.compile("^urn:cts:")

    def __init__(self, resolver):
        self.__resolver__ = resolver

    @staticmethod
    def match(uri):
        """ Check to see if this URI is retrievable by this Retriever implementation

        :param uri: the URI of the resource to be retrieved
        :type uri: str
        :return: True if it can be, False if not
        :rtype: bool
        """
        # prototype implementation can't retrieve anything!
        return CTSRetriever.__reg_exp__.match(uri) is not None

    def read(self, uri):
        """ Retrieve the contents of the resource

        :param uri: the URI of the resource to be retrieved
        :type uri: str
        :return: the contents of the resource
        :rtype: str
        """
        return self.__resolver__.getTextualNode(uri).export(Mimetypes.XML.TEI), "text/xml"



# -*- coding: utf-8 -*-

from flask_nemo.plugin import PluginPrototype
from flask_nemo.filters import f_slugify
from pkg_resources import resource_filename


class Breadcrumb(PluginPrototype):
    """
        The Breadcrumb plugin is enabled by default in Nemo.
        It can be overwritten or removed. It simply adds a breadcrumb

    """
    HAS_AUGMENT_RENDER = True
    TEMPLATES = {"main": resource_filename("flask_nemo", "data/templates_plugins/breadcrumb")}

    def render(self, **kwargs):
        """ Make breadcrumbs for a route

        :param kwargs: dictionary of named arguments used to construct the view
        :type kwargs: dict
        :return: List of dict items the view can use to construct the link.
        :rtype: {str: list({ "link": str, "title", str, "args", dict})}
        """
        breadcrumbs = []
        # this is the list of items we want to accumulate in the breadcrumb trail.
        # item[0] is the key into the kwargs["url"] object and item[1] is the  name of the route
        # setting a route name to None means that it's needed to construct the route of the next item in the list
        # but shouldn't be included in the list itself (this is currently the case for work --
        # at some point we probably should include work in the navigation)
        breadcrumbs = []
        if "collections" in kwargs:
            breadcrumbs = [{
                "title": "Text Collections",
                "link": ".r_collections",
                "args": {}
            }]

            if "parents" in kwargs["collections"]:
                breadcrumbs += [
                        {
                            "title": parent["label"],
                            "link": ".r_collection_semantic",
                            "args": {
                                "objectId": parent["id"],
                                "semantic": f_slugify(parent["label"]),
                            },
                        }
                        for parent in kwargs["collections"]["parents"]
                  ][::-1]

            if "current" in kwargs["collections"]:
                breadcrumbs.append({
                    "title": kwargs["collections"]["current"]["label"],
                    "link": None,
                    "args": {}
                })

        # don't link the last item in the trail
        if len(breadcrumbs) > 0:
            breadcrumbs[-1]["link"] = None

        return {"breadcrumbs": breadcrumbs}



from jinja2 import nodes
from jinja2.ext import Extension


class FakeCacheExtension(Extension):
    """ This extension exists only to avoid breaks in Nemo if FlaskCache is not used.
    Not that if you'd load Flask Cache en

    """
    tags = set(['cache'])

    def parse(self, parser):
        lineno = next(parser.stream).lineno

        #: Parse timeout
        args = [parser.parse_expression()]

        #: Parse fragment name
        #: Grab the fragment name if it exists
        #: otherwise, default to the old method of using the templates
        #: lineno to maintain backwards compatibility.
        if parser.stream.skip_if('comma'):
            args.append(parser.parse_expression())
        else:
            args.append(nodes.Const("%s%s" % (parser.filename, lineno)))

        #: Parse vary_on parameters
        vary_on = []
        while parser.stream.skip_if('comma'):
            vary_on.append(parser.parse_expression())

        if vary_on:
            args.append(nodes.List(vary_on))
        else:
            args.append(nodes.Const([]))

        body = parser.parse_statements(['name:endcache'], drop_needle=True)
        return nodes.CallBlock(self.call_method('_do_nothing', args),
                               [], [], body).set_lineno(lineno)

    def _do_nothing(self, timeout, fragment_name, vary_on, caller):
        return caller()



from collections import OrderedDict
from flask_nemo.common import join_or_single


def default_chunker(text, getreffs):
    """ This is the default chunker which will resolve the reference giving a callback (getreffs) and a text object with its metadata

    :param text: Text Object representing either an edition or a translation
    :type text: MyCapytains.resources.inventory.Text
    :param getreffs: callback function which retrieves a list of references
    :type getreffs: function

    :return: List of urn references with their human readable version
    :rtype: [(str, str)]
    """
    level = len(text.citation)
    return [tuple([reff.split(":")[-1]]*2) for reff in getreffs(level=level)]


def scheme_chunker(text, getreffs):
    """ This is the scheme chunker which will resolve the reference giving a callback (getreffs) and a text object with its metadata

    :param text: Text Object representing either an edition or a translation
    :type text: MyCapytains.resources.inventory.Text
    :param getreffs: callback function which retrieves a list of references
    :type getreffs: function

    :return: List of urn references with their human readable version
    :rtype: [(str, str)]
    """
    level = len(text.citation)
    types = [citation.name for citation in text.citation]
    if types == ["book", "poem", "line"]:
        level = 2
    elif types == ["book", "line"]:
        return line_chunker(text, getreffs)
    return [tuple([reff.split(":")[-1]]*2) for reff in getreffs(level=level)]


def line_chunker(text, getreffs, lines=30):
    """ Groups line reference together

    :param text: Text object
    :type text: MyCapytains.resources.text.api
    :param getreffs: Callback function to retrieve text
    :type getreffs: function(level)
    :param lines: Number of lines to use by group
    :type lines: int
    :return: List of grouped urn references with their human readable version
    :rtype: [(str, str)]
    """
    level = len(text.citation)
    source_reffs = [reff.split(":")[-1] for reff in getreffs(level=level)]
    reffs = []
    i = 0
    while i + lines - 1 < len(source_reffs):
        reffs.append(tuple([source_reffs[i]+"-"+source_reffs[i+lines-1], source_reffs[i]]))
        i += lines
    if i < len(source_reffs):
        reffs.append(tuple([source_reffs[i]+"-"+source_reffs[len(source_reffs)-1], source_reffs[i]]))
    return reffs


def level_chunker(text, getValidReff, level=1):
    """ Chunk a text at the passage level

    :param text: Text object
    :type text: MyCapytains.resources.text.api
    :param getreffs: Callback function to retrieve text
    :type getreffs: function(level)
    :return: List of urn references with their human readable version
    :rtype: [(str, str)]
    """
    references = getValidReff(level=level)
    return [(ref.split(":")[-1], ref.split(":")[-1]) for ref in references]


def level_grouper(text, getValidReff, level=None, groupby=20):
    """ Alternative to level_chunker: groups levels together at the latest level

    :param text: Text object
    :param getValidReff: GetValidReff query callback
    :param level: Level of citation to retrieve
    :param groupby: Number of level to groupby
    :return: Automatically curated references
    """
    if level is None or level > len(text.citation):
        level = len(text.citation)

    references = [ref.split(":")[-1] for ref in getValidReff(level=level)]
    _refs = OrderedDict()

    for key in references:
        k = ".".join(key.split(".")[:level-1])
        if k not in _refs:
            _refs[k] = []
        _refs[k].append(key)
        del k

    return [
        (
            join_or_single(ref[0], ref[-1]),
            join_or_single(ref[0], ref[-1])
        )
        for sublist in _refs.values()
        for ref in [
            sublist[i:i+groupby]
            for i in range(0, len(sublist), groupby)
        ]
    ]



from setuptools import setup, find_packages

version = "1.0.0b4"

setup(
    name='flask_nemo',
    version=version,
    packages=find_packages(exclude=["examples", "tests"]),
    url='https://github.com/capitains/flask-capitains-nemo',
    license='GNU GPL',
    author='Thibault Clerice',
    author_email='leponteineptique@gmail.com',
    description='Flask Extension to browse CTS Repository',
    test_suite="tests",
    install_requires=[
        "MyCapytain>=2.0.0b5",
        "requests_cache>=0.4.9",
        "Flask>=0.12",
        "requests>=2.10.0",
        "python-slugify==1.2.1",
        "Flask-Caching>=1.2.0"
    ],
    tests_require=[
        "mock>=2.0.0",
    ],
    entry_points={
        'console_scripts': ['capitains-nemo=flask_nemo.cmd:server'],
    },
    include_package_data=True,
    zip_safe=False
)



from flask import Flask
from flask_nemo import Nemo
from flask_nemo.plugin import PluginPrototype
from flask_nemo.filters import f_formatting_passage_reference
from tests.test_resources import NemoResource


class FilterPlugin(PluginPrototype):
    FILTERS = ["f_make_string"]

    def f_make_string(self, number):
        return str(number)


class FilterPluginOverride(PluginPrototype):
    FILTERS = ["f_formatting_passage_reference"]

    def f_formatting_passage_reference(self, number):
        return str(number)


class TestPluginFilters(NemoResource):
    """ Test plugin implementation of filters
    """

    def test_register_filter(self):
        app = Flask(__name__)
        plug = FilterPlugin()
        self.nemo = Nemo(app=app, plugins=[plug])
        self.assertEqual(
            self.nemo.app.jinja_env.filters["formatting_passage_reference"],
            f_formatting_passage_reference,
            "Original Nemo filters should be kept"
        )
        self.assertEqual(
            self.nemo.app.jinja_env.filters["make_string"],
            plug.f_make_string,
            "Plugin filters should be registered"
        )

    def test_register_filter_override(self):
        app = Flask(__name__)
        plug = FilterPluginOverride()
        self.nemo = Nemo(app=app, plugins=[plug])
        self.assertEqual(
            self.nemo.app.jinja_env.filters["formatting_passage_reference"],
            plug.f_formatting_passage_reference,
            "Original Nemo filters should be override by newly registered filters"
        )

    def test_register_filter_namespace(self):
        app = Flask(__name__)

        plug = FilterPluginOverride(namespacing=True, name="no_override")
        plug_noname = FilterPluginOverride(namespacing=True)

        self.nemo = Nemo(app=app, plugins=[plug, plug_noname])
        self.assertEqual(
            self.nemo.app.jinja_env.filters["formatting_passage_reference"],
            f_formatting_passage_reference,
            "Original Nemo filters should be kept when namespacing is done"
        )
        self.assertEqual(
            self.nemo.app.jinja_env.filters["no_override_formatting_passage_reference"],
            plug.f_formatting_passage_reference,
            "Original Nemo filters should be override by newly registered filters"
        )
        self.assertEqual(
            self.nemo.app.jinja_env.filters["FilterPluginOverride_formatting_passage_reference"],
            plug_noname.f_formatting_passage_reference,
            "Original Nemo filters should be override by newly registered filters"
        )


from flask_nemo import Nemo
from flask_nemo.chunker import level_grouper
from flask import Flask
from MyCapytain.resolvers.cts.api import HttpCTSResolver
from MyCapytain.resolvers.cts.local import CTSCapitainsLocalResolver
from MyCapytain.retrievers.cts5 import CTS
import argparse
import sys


class Server:
    @staticmethod
    def runner(method, address, port, host, css, xslt, groupby, debug):
        resolver = None
        app = Flask(
            __name__
        )
        if method == "cts-api":
            resolver = HttpCTSResolver(CTS(address))
        elif method == "cts-local":
            resolver = CTSCapitainsLocalResolver([address])
        if xslt is not None:
            xslt = {"default": xslt}
        # We set up Nemo
        nemo = Nemo(
            app=app,
            name="nemo",
            base_url="",
            css=css,
            transform=xslt,
            resolver=resolver,
            chunker={"default": lambda x, y: level_grouper(x, y, groupby=groupby)}
        )

        # We run the app
        app.debug = debug
        app.run(port=port, host=host)
        # For test purposes
        return nemo, app

    @staticmethod
    def parser(args):
        parser = argparse.ArgumentParser(
            description="""Capitains Nemo UI
        Currently, can be used with either a CTS API Address or a local CapiTainS repository"""
        )
        parser.add_argument('method', type=str, choices=["cts-api", "cts-local"],
                           help='Method to retrieve resource')
        parser.add_argument('address', type=str, default=None,
                           help="""Path or address of the resource required by method
    - Local CapiTainS repository [ http://capitains.github.io/pages/guidelines ]
    - HTTP CTS Address
    """)
        parser.add_argument('--port', type=int, default=8000,
                           help='Port to use for the HTTP Server')
        parser.add_argument('--host', type=str, default="127.0.0.1",
                           help='Host to use for the HTTP Server')
        parser.add_argument('--css', type=str, default=None, nargs='*',
                           help='Full path to secondary css file')
        parser.add_argument('--xslt', type=str, default=None,
                           help='Default XSLT to use')
        parser.add_argument('--groupby', type=int, default=25,
                           help='Number of passage to group in the deepest level of the hierarchy')
        parser.add_argument('--debug', action="store_true", default=False, help="Set-up the application for debugging")

        args = vars(parser.parse_args(args))
        print("Running with {}".format(" ".join(["{}={}".format(k, v) for k, v in args.items()])))
        return Server.runner(**args)

    @staticmethod
    def cmd():
        return Server.parser(sys.argv[1:])

if __name__ == "__main__":
    Server.cmd()



"""

Test only controllers functions, ie function that should not be accessible directly from http routes but are still
greatly tied to the app/blueprint instance

"""
from flask.ext.nemo import Nemo
from mock import patch, call, Mock
from lxml import etree
from flask import Markup, Flask

from MyCapytain.resources.prototypes.text import Passage
from MyCapytain.common.constants import Mimetypes
from MyCapytain.resources.prototypes.cts.inventory import PrototypeTextInventory, PrototypeText

from tests.test_resources import NemoResource
from flask_nemo.chunker import level_grouper
from tests.test_resources import NautilusDummy


class NemoTestControllers(NemoResource):

    def test_flask_nemo(self):
        """ Testing Flask Nemo is set up"""
        a = Nemo()
        self.assertIsInstance(a, Nemo)
        a = Nemo()

    def test_without_inventory_request(self):
        """ Check that endpoint are derived from nemo.api_endpoint setting
        """
        #  Test without inventory
        with patch('requests.get', return_value=self.getCapabilities) as patched_get:
            self.nemo.get_inventory()
            patched_get.assert_called_once_with(
                NemoTestControllers.endpoint.endpoint.endpoint, params={
                    "request": "GetCapabilities"
                }
            )

    def test_inventory_parsing(self):
        """ Check that endpoint request leads to the creation of a TextInventory object
        """
        with patch('requests.get', return_value=self.getCapabilities) as patched_get:
            inventory = self.nemo.get_inventory()
            patched_get.assert_called_once_with(
                NemoTestControllers.endpoint.endpoint.endpoint, params={
                    "request": "GetCapabilities"
                }
            )
            self.assertIs(len(inventory.textgroups), 4)

    def test_get_authors(self):
        """ Check that authors textgroups are returned with informations
        """
        with patch('requests.get', return_value=self.getCapabilities):
            tgs = self.nemo.get_inventory()
            self.assertIs(len(tgs.members), 4)
            self.assertEqual("urn:cts:greekLit:tlg0003" in [str(tg.id) for tg in tgs.members], True)

    def test_get_works_with_collection(self):
        """ Check that works are returned when filtered by collection and textgroup
        """
        with patch('requests.get', return_value=self.getCapabilities):
            works = self.nemo.get_collection("urn:cts:greekLit:tlg0003")
            self.assertIs(len(works), 1)
            self.assertIn(
                "urn:cts:greekLit:tlg0003.tlg001",
                [str(w.id) for w in works.descendants]
            )

            #  Check when it fails
            with self.assertRaises(KeyError):
                works = self.nemo.get_collection("urn:cts:greekLit:tlg000b3")

    def test_get_texts_with_all_filtered(self):
        """ Check that texts are filtered
        """
        with patch('requests.get', return_value=self.getCapabilities):
            texts = self.nemo.get_collection("urn:cts:greekLit:tlg0003.tlg001")
            self.assertIs(len(texts), 1)
            self.assertIn(
                "urn:cts:greekLit:tlg0003.tlg001.perseus-grc2",
                [str(text.id) for text in texts.descendants]
            )

            with self.assertRaises(KeyError):
                texts = self.nemo.get_collection("urn:cts:greekLit:tlg0003.tlg002")
                self.assertIs(len(texts), 0)

    def test_get_texts_with_work_not_filtered(self):
        """ Check that all textgroups texts are returned
        """
        with patch('requests.get', return_value=self.getCapabilities):
            texts = self.nemo.get_collection("urn:cts:latinLit:phi0959").readableDescendants
            self.assertIs(len(texts), 10)
            self.assertNotIn(
                "urn:cts:greekLit:tlg0003.tlg001.perseus-grc2",
                [str(text.id) for text in texts]
            )
            self.assertNotIn(
                "urn:cts:latinLit:phi0959.tlg001.perseus-lat2",
                [str(text.id) for text in texts]
            )

    def test_get_texts_raise(self):
        with patch('requests.get', return_value=self.getCapabilities):
            with self.assertRaises(KeyError):
                self.nemo.get_collection("latinLit")

    def test_get_single_text(self):
        """ Check that texts are filtered
        """
        with patch('requests.get', return_value=self.getCapabilities):
            texts = self.nemo.get_collection("urn:cts:greekLit:tlg0003.tlg001.perseus-grc2")
            self.assertIsInstance(texts, PrototypeText)
            self.assertEqual(str(texts.id), "urn:cts:greekLit:tlg0003.tlg001.perseus-grc2")

    def test_get_single_text_empty_because_no_work(self):
        """ Check that texts are filtered
        """
        with patch('requests.get', return_value=self.getCapabilities):
            with self.assertRaises(KeyError):
                texts = self.nemo.get_collection("urn:cts:latinLit:phi0959.phi011.perseus-lat2")

    def test_get_single_text_abort(self):
        """ Check that texts are filtered
        """
        with patch('requests.get', return_value=self.getCapabilities):
            with self.assertRaises(KeyError):
                texts = self.nemo.get_collection("urn:cts:greekLIT:TLG0003:tlg001:perseus-grc132")

    def test_get_validreffs_without_specific_callback(self):
        """ Try to get valid reffs
        """
        with patch('requests.get', return_value=self.getValidReff) as patched:
            text, reffs = self.nemo.get_reffs(
                "urn:cts:latinLit:phi1294.phi002.perseus-lat2",
                export_collection=True
            )
            self.assertIsInstance(text, PrototypeText)
            self.assertIsInstance(reffs, list)
            self.assertEqual(reffs[0], ("1.pr.1-1.pr.20", "1.pr.1-1.pr.20"))
            self.assertEqual(patched.mock_calls, [
                call(
                    NemoTestControllers.endpoint.endpoint.endpoint,
                    params={
                        "request": "GetCapabilities"
                    }
                ),
                call(
                    NemoTestControllers.endpoint.endpoint.endpoint,
                    params={
                        "request": "GetValidReff",
                        "level": "3",
                        "urn": "urn:cts:latinLit:phi1294.phi002.perseus-lat2"
                    }
                )
                ]
            )

    def test_get_passage(self):
        with patch('requests.get', return_value=self.getPassagePlus) as patched:
            passage = self.nemo.get_passage(
                "urn:cts:latinLit:phi1294.phi002.perseus-lat2",
                "1.pr"
            )
            self.assertIsInstance(passage, Passage)
            self.assertEqual(
                len(passage.export(Mimetypes.PYTHON.ETREE)
                    .xpath("//tei:l[@n]", namespaces={"tei": "http://www.tei-c.org/ns/1.0"})),
                6,
                "There should be six lines"
            )

    def test_get_siblings(self):
        """ Test first passage
        """
        app = Flask("Nemo")
        nemo = Nemo(
            app=app,
            base_url="",
            resolver=NautilusDummy,
            original_breadcrumb=False,
            chunker={"default": lambda x, y: level_grouper(x, y, groupby=20)}
        )
        # First range
        p = nemo.get_passage("urn:cts:latinLit:phi1294.phi002.perseus-lat2", "1.pr.1-1.pr.20")
        l, r = nemo.get_siblings("urn:cts:latinLit:phi1294.phi002.perseus-lat2", "1.pr.1-1.pr.20", p)
        self.assertEqual(l, None, "There should be nothing on the left")
        self.assertEqual(r, "1.pr.21-1.pr.22", "Next passage should not be 20 long")
        # Second range
        p = nemo.get_passage("urn:cts:latinLit:phi1294.phi002.perseus-lat2", "1.pr.21-1.pr.22")
        l, r = nemo.get_siblings("urn:cts:latinLit:phi1294.phi002.perseus-lat2", "1.pr.21-1.pr.22", p)
        self.assertEqual(l, "1.pr.1-1.pr.20", "There should be something on the left")
        self.assertEqual(r, "1.1.1-1.1.6", "Next passage should not be 20 long")
        # Last range
        p = nemo.get_passage("urn:cts:latinLit:phi1294.phi002.perseus-lat2", "14.223.1-14.223.2")
        l, r = nemo.get_siblings("urn:cts:latinLit:phi1294.phi002.perseus-lat2", "14.223.1-14.223.2", p)
        self.assertEqual(l, "14.222.1-14.222.2", "Previous passage does not have to be 20 long")
        self.assertEqual(r, None, "There should be nothing on the right")
        # Passage with unknown range
        p = nemo.get_passage("urn:cts:latinLit:phi1294.phi002.perseus-lat2", "1.11.1-1.11.2")
        l, r = nemo.get_siblings("urn:cts:latinLit:phi1294.phi002.perseus-lat2", "1.11.1-1.11.2", p)
        self.assertEqual(l, "1.10.3-1.10.4", "Passage should be computed specifically if texts have unknown range")
        self.assertEqual(r, "1.11.3-1.11.4", "Passage should be computed specifically if texts have unknown range")



import json
from tests.test_plugin.test_resources import make_client
from unittest import TestCase
from flask_nemo.plugins.annotations_api import AnnotationsApiPlugin
from flask_nemo.query.proto import QueryPrototype
from flask_nemo.query.annotation import AnnotationResource
from flask import Response


class MockQueryInterface(QueryPrototype):
    ANNOTATION = AnnotationResource(
        "uri", ("urn:cts:latinLit:phi1294.phi002.perseus-lat2", "1"), "http://foo.bar/treebank",
        resolver=None,  # Overwriting this one for the purpose of the test
        mimetype="application/xml", slug="treebank"
    )
    ANNOTATION2 = AnnotationResource(
        "uri2", ("urn:cts:latinLit:phi1294.phi002.perseus-lat2", "2"), "http://foo.bar/treebank",
        resolver=None,  # Overwriting this one for the purpose of the test
        mimetype="application/json", slug="treebank"
    )

    def getAnnotations(self, targets, wildcard=".", include=None, exclude=None, limit=None, start=1, expand=False,
                       **kwargs):
        if targets == None:
            return 2, [
                type(self).ANNOTATION,
                type(self).ANNOTATION2
            ]
        else:
            return 1, [
                type(self).ANNOTATION
            ]

    def getResource(self, sha):
        annotation = type(self).ANNOTATION
        if sha == "abc":
            annotation.read = lambda: Response("a", headers={"Content-Type": "text/plain"})
            annotation.read = lambda: Response("a", headers={"Content-Type": "text/plain"})
        else:
            annotation.__mimetype__ = "application/xml"
            annotation.read = lambda: "123"
        return annotation


class RetrieverMock(object):
    @staticmethod
    def match(uri):
        return True

    @staticmethod
    def read(uri):
        return "{} has been read".format(uri), "mimetype"


class AnnotationsApiPluginDummyInterfaceTest(TestCase):
    """ Test Suite for Annotations Api Plugin with a mock of query interface
    """

    def setUp(self):
        self.ann_plugin = AnnotationsApiPlugin(name="testplugin", queryinterface=MockQueryInterface(lambda x: x))
        self.client = make_client(self.ann_plugin)
        self.maxDiff = 50000

    def test_route_by_target(self):
        """ Check error response for improper urn
        """
        response = self.client.get("/api/annotations?target=urn:cts:latinLit:phi1294.phi002.perseus-lat2:1")
        data = json.loads(response.data.decode("utf-8"))
        self.assertEqual(
            data,
            {
              "@context": {
                "": "http://www.w3.org/ns/anno.jsonld",
                "dc": "http://purl.org/dc/terms/",
                "owl": "http://www.w3.org/2002/07/owl#",
                'nemo': 'https://capitains.github.io/flask-capitains-nemo/ontology/#'
              },
              "id": "/api/annotations?start=1",
              "startIndex": 1,
              "total": 1,
              "type": "AnnotationCollection",
              "items": [
                {
                  "body": "/api/annotations/922fffd1bd6fdaa95b4545df7a78754f6d67c2272b2900aa3ccd5e9da3dbb179/body",
                  "id": "/api/annotations/922fffd1bd6fdaa95b4545df7a78754f6d67c2272b2900aa3ccd5e9da3dbb179",
                  "target": {
                    "source": "urn:cts:latinLit:phi1294.phi002.perseus-lat2",
                    "selector": {
                        "type": "FragmentSelector",
                        "conformsTo": "http://ontology-dts.org/terms/subreference",
                        "value": "1"
                    }
                  },
                  "type": "Annotation",
                  "dc:type": "http://foo.bar/treebank",
                  "nemo:slug": "treebank",
                  "owl:sameAs": [
                    "uri"
                  ]
                }
              ]
            },
            "Response of the api should work with default annotation"
        )
        self.assertEqual(200, response.status_code, "HTTP Code for valid request is 200")

    def test_all_annotation(self):
        """ Check error response for improper urn
        """
        response = self.client.get("/api/annotations")
        data = json.loads(response.data.decode("utf-8"))
        self.assertEqual(
            data,
            {
              "@context": {
                "": "http://www.w3.org/ns/anno.jsonld",
                "dc": "http://purl.org/dc/terms/",
                "owl": "http://www.w3.org/2002/07/owl#",
                'nemo': 'https://capitains.github.io/flask-capitains-nemo/ontology/#'
              },
              "id": "/api/annotations?start=1",
              "startIndex": 1,
              "total": 2,
              "type": "AnnotationCollection",
              "items": [
                {
                  "body": "/api/annotations/922fffd1bd6fdaa95b4545df7a78754f6d67c2272b2900aa3ccd5e9da3dbb179/body",
                  "id": "/api/annotations/922fffd1bd6fdaa95b4545df7a78754f6d67c2272b2900aa3ccd5e9da3dbb179",
                  "target": {
                    "source": "urn:cts:latinLit:phi1294.phi002.perseus-lat2",
                    "selector": {
                        "type": "FragmentSelector",
                        "conformsTo": "http://ontology-dts.org/terms/subreference",
                        "value": "1"
                    }
                  },
                  "type": "Annotation",
                  "dc:type": "http://foo.bar/treebank",
                  "nemo:slug": "treebank",
                  "owl:sameAs": [
                    "uri"
                  ]
                },
                {
                  "body": "/api/annotations/15c0c44af733054c9df420d046b871ef2263df0f1181b663bd7ba04c05032509/body",
                  "id": "/api/annotations/15c0c44af733054c9df420d046b871ef2263df0f1181b663bd7ba04c05032509",
                  "target": {
                    "source": "urn:cts:latinLit:phi1294.phi002.perseus-lat2",
                    "selector": {
                        "type": "FragmentSelector",
                        "conformsTo": "http://ontology-dts.org/terms/subreference",
                        "value": "2"
                    }
                  },
                  "type": "Annotation",
                  "dc:type": "http://foo.bar/treebank",
                  "nemo:slug": "treebank",
                  "owl:sameAs": [
                    "uri2"
                  ]
                }
              ]
            },
            "Response of the api should work with default annotation"
        )
        self.assertEqual(200, response.status_code, "HTTP Code for valid request is 200")

    def test_route_get_content_response(self):
        """ Check error response for improper urn
        """
        response = self.client.get("/api/annotations/abc/body")
        data = response.data.decode("utf-8")
        self.assertEqual(
            data,
            "a",
            "Content of the response should be forwarded"
        )
        self.assertEqual("text/plain", response.headers["Content-Type"], "Response mimetype should be forwarded")

    def test_route_get_content(self):
        """ Check error response for improper urn
        """
        response = self.client.get("/api/annotations/abc")
        data = response.data.decode("utf-8")
        self.assertEqual(
            json.loads(data),
            {
                "@context": {
                    "": "http://www.w3.org/ns/anno.jsonld",
                    "dc": "http://purl.org/dc/terms/",
                    "owl": "http://www.w3.org/2002/07/owl#",
                    'nemo': 'https://capitains.github.io/flask-capitains-nemo/ontology/#'
                },
                "body": "/api/annotations/922fffd1bd6fdaa95b4545df7a78754f6d67c2272b2900aa3ccd5e9da3dbb179/body",
                "id": "/api/annotations/922fffd1bd6fdaa95b4545df7a78754f6d67c2272b2900aa3ccd5e9da3dbb179",
                "target": {
                    "source": "urn:cts:latinLit:phi1294.phi002.perseus-lat2",
                    "selector": {
                        "type": "FragmentSelector",
                        "conformsTo": "http://ontology-dts.org/terms/subreference",
                        "value": "1"
                    }
                },
                "type": "Annotation",
                "dc:type": "http://foo.bar/treebank",
                "nemo:slug": "treebank",
                "owl:sameAs": [
                    "uri"
                ]
            },
            "Annotation route should return "
        )
        self.assertEqual("application/json", response.headers["Content-Type"], "Response should be json")

    def test_route_get_content_normal(self):
        """ Check error response for improper urn
        """
        response = self.client.get("/api/annotations/def/body")
        self.assertEqual(
            response.data,
            b"123",
            "Content of the response should be forwarded"
        )
        self.assertEqual("application/xml", response.headers["Content-Type"], "Read() mimetype should be forwarded")


class AnnotationsApiPluginTest(TestCase):
    """ Test Suite for Annotations Api Plugin in empty case
    """

    def setUp(self):
        self.ann_plugin = AnnotationsApiPlugin(name="testplugin", queryinterface=QueryPrototype(lambda x: x))
        self.client = make_client(self.ann_plugin)

    def test_route_by_target_valid_urn(self):
        """ Check empty response for valid urn target (given prototype query interface which knows nothing)
        """
        response = self.client.get("/api/annotations?target=urn:cts:greekLit:tlg0012.tlg001.grc1")
        self.assertEqual({"annotations": [], "count": 0}, json.loads(response.get_data().decode("utf-8")))
        self.assertEqual(200, response.status_code)

    def test_route_by_target_invalid_urn(self):
        """ Check error response for improper urn
        """
        response = self.client.get("/api/annotations?target=foo")
        self.assertEqual(b"invalid urn", str(response.data))
        self.assertEqual(400, response.status_code)

    def test_route_get(self):
        """ Check not found response for valid urn target (given prototype query interface which knows nothing)
        """
        response = self.client.get("/api/annotations/foo")
        self.assertEqual("b'invalid resource uri'", str(response.data))
        self.assertEqual(404, response.status_code)

    def test_route_get_body(self):
        """ Check not found response for valid urn target (given prototype query interface which knows nothing)
        """
        response = self.client.get("/api/annotations/foo/body")
        self.assertEqual("b'invalid resource uri'", str(response.data))
        self.assertEqual(404, response.status_code)


class AnnotationsApiPluginTest(TestCase):
    """ Test Suite for Annotations Api Plugin in empty case
    """

    def setUp(self):
        self.ann_plugin = AnnotationsApiPlugin(name="testplugin", queryinterface=QueryPrototype(lambda x: x))
        self.client = make_client(self.ann_plugin)

    def test_route_by_target_valid_urn(self):
        """ Check empty response for valid urn target (given prototype query interface which knows nothing)
        """
        response = self.client.get("/api/annotations?target=urn:cts:greekLit:tlg0012.tlg001.grc1")
        self.maxDiff = 20000
        self.assertEqual(
            json.loads(response.get_data().decode("utf-8")),
            {
                '@context': {
                    '': 'http://www.w3.org/ns/anno.jsonld',
                    'dc': 'http://purl.org/dc/terms/',
                    'owl': 'http://www.w3.org/2002/07/owl#',
                    'nemo': 'https://capitains.github.io/flask-capitains-nemo/ontology/#'
                },
                'id': '/api/annotations?start=1',
                'items': [],
                'startIndex': 1,
                'total': 0,
                'type': 'AnnotationCollection'
             }
        )
        self.assertEqual(200, response.status_code)

    def test_route_by_target_invalid_urn(self):
        """ Check error response for improper urn
        """
        response = self.client.get("/api/annotations?target=foo")
        self.assertEqual("b'invalid urn'", str(response.data))
        self.assertEqual(400, response.status_code)

    def test_route_get(self):
        """ Check not found response for valid urn target (given prototype query interface which knows nothing)
        """
        response = self.client.get("/api/annotations/foo")
        self.assertEqual("b'invalid resource uri'", str(response.data))
        self.assertEqual(404, response.status_code)

    def test_route_get_body(self):
        """ Check not found response for valid urn target (given prototype query interface which knows nothing)
        """
        response = self.client.get("/api/annotations/foo/body")
        self.assertEqual("b'invalid resource uri'", str(response.data))
        self.assertEqual(404, response.status_code)

